﻿using System;
using System.Collections.Generic;
using System.Diagnostics;

namespace TSLab.Script.Helpers
{
    /// <summary>
    /// \~english Some indicator's calculation helpers \~russian Помощники для расчета индикаторов
    /// </summary>
    public static class Indicators
    {
        /// <summary>
        /// \~english Calculate highest value \~russian Рассчитать максимальное значение
        /// </summary>
        /// <param name="curbar">\~english last calculation bar \~russian Последний рассчитанный бар</param>
        /// <param name="period">\~english calculation period \~russian Период расчета</param>
        /// <param name="candles">\~english source candles list \~russian Список источников баров</param>
        /// <returns>\~english highest value \~russian Максимальное значение</returns>
        public static double Highest(IList<double> candles, int curbar, int period)
        {
            double high = Double.MinValue;
            int start = curbar - period + 1;
            if (start < 0)
            {
                start = 0;
            }
            for (int i = start; i <= curbar; i++)
            {
                high = Math.Max(high, candles[i]);
            }
            return high;
        }

        /// <summary>
        /// \~english Calculate lowest value \~russian Рассчитать минимальное значение
        /// </summary>
        /// <param name="curbar">\~english last calculation bar \~russian Последний рассчитанный бар</param>
        /// <param name="period">\~english calculation period \~russian Период расчета</param>
        /// <param name="candles">\~english source candles list \~russian Список источников баров</param>
        /// <returns>\~english lowest value \~russian Минимальное значение</returns>
        public static double Lowest(IList<double> candles, int curbar, int period)
        {
            double low = Double.MaxValue;
            int start = curbar - period + 1;
            if (start < 0)
            {
                start = 0;
            }
            for (int i = start; i <= curbar; i++)
            {
                low = Math.Min(low, candles[i]);
            }
            return low;
        }

        /// <summary>
        /// \~english Calculate True Range for current bar \~russian Рассчитать истинный диапазон для текущего бара
        /// </summary>
        /// <param name="candles">\~english source candles list \~russian Список источников баров</param>
        /// <param name="curbar">\~english current bar number \~russian Номер текущего бара</param>
        /// <returns>\~english True Range \~russian Истинный диапазон</returns>
        public static double TrueRange(IList<Bar> candles, int curbar)
        {
            Bar bar = candles[curbar];
            double high = bar.High;
            double low = bar.Low;
            double tr = Math.Abs(high - low);
            if (curbar > 0)
            {
                double oldclose = candles[curbar - 1].Close;
                tr = Math.Max(tr, Math.Abs(oldclose - high));
                tr = Math.Max(tr, Math.Abs(oldclose - low));
            }
            return tr;
        }

        /// <summary>
        /// \~english Calculate SMA for current bar \~russian Рассчитать SMA (простое скользящее среднее) для текущего бара
        /// </summary>
        /// <param name="candles">\~english source candles list \~russian Список источников баров</param>
        /// <param name="curbar">\~english current bar number \~russian Номер текущего бара </param>
        /// <param name="period">\~english SMA period \~russian Период SMA (простого скользящего среднего)</param>
        /// <returns>\~english SMA \~russian SMA (Простое cкользящее cреднее)</returns>
        public static double SMA(IList<double> candles, int curbar, int period)
        {
            int start = curbar - period + 1;
            if (start < 0)
            {
                start = 0;
            }
            period = curbar - start + 1;
            double sum = 0;
            while (start <= curbar && start < candles.Count)
            {
                sum += candles[start++];
            }
            sum /= Math.Min(period, curbar + 1);
            Debug.Assert(!double.IsNaN(sum));
            return sum;
        }

        /// <summary>
        /// \~english Calculate Summ for same values\~russian Рассчитать сумму за несколько значений
        /// </summary>
        /// <param name="candles">\~english source candles list \~russian Список источников баров</param>
        /// <param name="curbar">\~english current bar number \~russian Номер текущего бара </param>
        /// <param name="period">\~english the period \~russian Период </param>
        /// <returns>\~english the Summ \~russian Сумма</returns>
        public static double SummFor(IList<double> candles, int curbar, int period)
        {
            int start = curbar - period + 1;
            if (start < 0)
            {
                start = 0;
            }
            double sum = 0;
            while (start <= curbar && start < candles.Count)
            {
                sum += candles[start++];
            }
            Debug.Assert(!double.IsNaN(sum));
            return sum;
        }

        /// <summary>
        /// \~english Calculate Standart Debiation for current bar \~russian Рассчитать Стандартное Отклонение для текущего бара
        /// </summary>
        /// <param name="candles">\~english source list \~russian Список данных</param>
        /// <param name="SMAs">\~english calculated SMAs list \~russian Список посчитанных SMA</param>
        /// <param name="curbar">\~english current bar number \~russian Номер текущего бара </param>
        /// <param name="period">\~english SMA period \~russian Период SMA (простого скользящего среднего)</param>
        /// <returns>\~english SMA \~russian SMA (Простое cкользящее cреднее)</returns>
        public static double StDev(IList<double> candles, IList<double> SMAs, int curbar, int period)
        {
            int start = curbar - period + 1;
            if (start < 0)
            {
                start = 0;
            }
            period = curbar - start + 1;
            double sum = 0;
            while (start <= curbar && start < candles.Count)
            {
                var s = candles[start] - SMAs[curbar];
                sum += s*s;
                start++;
            }
            var n = Math.Min(period, curbar + 1);
            return Math.Sqrt(sum/n);
        }
    }
}